package wifi;

import java.net.NetworkInterface;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
//import android.content.IntentFilter;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.ScanResult;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Events;
import anywheresoftware.b4a.BA.Permissions;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;

@ShortName("MLwifi")
@Version(2.17f)
@Author("Jem Miller - Missing Link Software")
@Permissions(values = {"android.permission.ACCESS_NETWORK_STATE","android.permission.ACCESS_WIFI_STATE","android.permission.CHANGE_WIFI_STATE","android.permission.ACCESS_FINE_LOCATION","android.permission.INTERNET"})
public class MLwifi {

/**
 * Turns wifi on or off
 * 
 * Pass True to enable wifi or False to disable.
 * 
 * Example
 * dim wifi as MLwifi
 * 
 * wifi.EnableWifi(true)
 */
public void EnableWifi(boolean Enabled) {
	Context context = BA.applicationContext;
	
	WifiManager wifi = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
	wifi.setWifiEnabled(Enabled);
	
}

/**
 * Returns the current devices Android API (the version of Android running on the device.) as an integer such as 19, 21, 23, etc.
 * 
 */
public static int getAPI(){
int currentapiVersion = android.os.Build.VERSION.SDK_INT;

/*if (currentapiVersion >= android.os.Build.VERSION_CODES.LOLLIPOP){
    // Do something for lollipop and above versions
} else{
    // do something for phones running an SDK before lollipop
}*/

return currentapiVersion;
}

/**
 * Returns MAC address of the current interface
 *  
 * Returns  MAC address or empty string
 */
public static String MACAddress(final BA ba) {
	
	if (getAPI() >= android.os.Build.VERSION_CODES.M){
	try {
        String interfaceName = "wlan0";
        List<NetworkInterface> interfaces = Collections.list(NetworkInterface.getNetworkInterfaces());
        for (NetworkInterface intf : interfaces) {
            if (!intf.getName().equalsIgnoreCase(interfaceName)){
                continue;
            }

            byte[] mac = intf.getHardwareAddress();
            if (mac==null){
                return "";
            }

            StringBuilder buf = new StringBuilder();
            for (byte aMac : mac) {
                buf.append(String.format("%02X:", aMac));
            }
            if (buf.length()>0) {
                buf.deleteCharAt(buf.length() - 1);
            }
            return buf.toString();
        }
    } catch (Exception ex) { } // for now eat exceptions
    return "";
	}
	else{
	Context context = BA.applicationContext;
		
	WifiManager manager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
	
	WifiInfo info = manager.getConnectionInfo();
	
	return info.getMacAddress();
	}
	
}



/**
 * Get IP address from the wifi interface)
 * 
 * Returns  address or empty string
 */
public static String IpAddress(final BA ba) {
    Context context = BA.applicationContext;
    
	WifiManager wifiManager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
	WifiInfo wifiInfo = wifiManager.getConnectionInfo();
	
	int ipAddress = wifiInfo.getIpAddress();
	
	String ip = null;

	ip = String.format("%d.%d.%d.%d",
	(ipAddress & 0xff),
	(ipAddress >> 8 & 0xff),
	(ipAddress >> 16 & 0xff),
	(ipAddress >> 24 & 0xff));
	
	return ip;

}



/**
 * Returns the current SSID
 * 
 */
public String SSID() {
	Context context = BA.applicationContext;
	String st = "";
	
	WifiManager wifiManager = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
	WifiInfo wifiInfo = wifiManager.getConnectionInfo();
	
	st = wifiInfo.getSSID().replace('"', ' ');
	
	return st.trim();
	
}


/**
 * Returns the raw signal strength in decibels as an integer
 * 
 * Returns 0 on error
 */
public int WifiSignal(){
	Context context = BA.applicationContext;
	
	try{
	WifiManager wifiManager = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);
	int linkSpeed = wifiManager.getConnectionInfo().getRssi();
	
	return linkSpeed;
	}
	catch (Exception e) {
		return 0;
	}
}


/**
 * Returns an integer between 0 and 100 as a percentage of signal strength
 * 
 * Returns 0 on error.
 */
public static int WifiStrength(final BA ba){
	Context context = BA.applicationContext;
	
          try
          {
              WifiManager wifiManager = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);
              int rssi = wifiManager.getConnectionInfo().getRssi();
              int level = WifiManager.calculateSignalLevel(rssi, 10);
              int percentage = (int) ((level/10.0)*100);
              return percentage;
          }
          catch (Exception e)
          {
              return 0;
          }
}


/**
 * Checks if wifi is connected. This only checks for wifi connection to a router.
 * It does NOT test for internet availability. Use isOnLine() for that.
 * 
 * Returns True if wifi is connected or False if 2g, 3g, 4g, wimax or no connection.
 */
public boolean isWifiConnected(){
	Context context = BA.applicationContext;
	
	ConnectivityManager connManager = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
	NetworkInfo mWifi = connManager.getNetworkInfo(ConnectivityManager.TYPE_WIFI);

	return mWifi.isConnected();
}


/**
 * Checks for internet connection
 * 
 * This method does not check or care what type of network is used.
 * It can be wifi, 2g, 3g, 4g, wimax, etc. It just tests for internet connectivity.
 * 
 * Returns True if internet is available.
 */
public boolean isOnLine() {
	Context context = BA.applicationContext;
	
    ConnectivityManager connectivityManager 
          = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
    
    NetworkInfo activeNetworkInfo = connectivityManager.getActiveNetworkInfo();
    
    return activeNetworkInfo != null;
}

@Events(values = {"ScanDone(Results() As String, Count As Int)" })
@ShortName("MLScan") 
public static class MLWifiScanner {

	   WifiManager WifiObj;
	   WifiScanReceiver wifiReciever;
	   private String eventname = "";
	   protected List<ScanResult> Results = null;
	   BA _ba = null;
	   
	   /**
	    * A string array containing all scanned access points in the area of the device.
	    * These are comma separated entries consisting of the following:
	    * 
	    * SSID, Security type, signal strength, and "Saved" if this network was previously saved for use on this device.
	    */
	   public String wifis[];
	   
	   /**
	    * Initializes all objects and scans for wifi networks. Builds a string array of results (wifis) that can be accessed directly or with a call to updateWifiList.
	    * THIS METHOD SHOULD ONLY BE CALLED ONCE! 
	    * 
	    */
	   public void startScan(BA ba, String EventName) {
		   Context context = BA.applicationContext;
		   
		   if (_ba == null){
		   _ba = ba;
		   
	      WifiObj = (WifiManager) context.getSystemService("wifi");
	      eventname = EventName.toLowerCase();
	      
	      wifiReciever = new WifiScanReceiver(this);
	      
	      /*if (WifiObj.isWifiEnabled() && WifiObj != null) {
		      context.registerReceiver(wifiReciever, new IntentFilter(WifiManager.SCAN_RESULTS_AVAILABLE_ACTION));

	      WifiObj.startScan();
	      }*/
		   }
		   
	      if (!WifiObj.isWifiEnabled())
	    	  WifiObj.setWifiEnabled(true);
	      
	      updateWifiList(ba);
	      
	   }
	   
	   
	   /**
	    * Call only after startScan has already been called and a string array of Access Points has been built. This method returns a string array containing the access points from the most recent scan, including background scans done automatically by the Android WifiManager.	    * 
	    * 
	    * Calls the ScanDone event when a new list is ready. 
	    * 
	    * Returns the string array of comma separated wifi access points by SSID. Each entry may contain the following:
	    * SSID, Security type, Signal level (in DB's), and the word "Saved" if the network was previously saved for use on this device.
	    */
	   public void updateWifiList(BA ba) {
		   /*String ssid = "";
	    	 String tmp = "";*/
	    	 Context context = BA.applicationContext;
	    	 
		   if (WifiObj.isWifiEnabled() && WifiObj != null) {
			   
			   if (this.wifiReciever == null) {
			      this.wifiReciever = new WifiScanReceiver(this);
			   }
			      
		context.registerReceiver(wifiReciever, new IntentFilter(WifiManager.SCAN_RESULTS_AVAILABLE_ACTION));
			    			        
		      WifiObj.startScan();
		      }
		   
	         /*Results = WifiObj.getScanResults();
	         
	         wifis = new String[Results.size()];
	         
	         for(int i = 0; i < Results.size(); i++){
	        	 wifis[i] = Results.get(i).SSID;
	        	 ssid = wifis[i];
	        	 tmp = Results.get(i).capabilities;
	        	 int lev = Results.get(i).level;
	        	 
	        	 if (tmp.contains("WPA2")) 
		        	wifis[i] = wifis[i] + ",WPA2" + "," + String.valueOf(lev);
	        	 else
		         if (tmp.contains("WPA"))
		        	wifis[i] = wifis[i] + ",WPA" + "," + String.valueOf(lev);
		        else
		        if (tmp.contains("WEP"))
		        	wifis[i] = wifis[i] + ",WEP" + "," + String.valueOf(lev);
		        else
		        	wifis[i] = wifis[i] + ",NONE" + "," + String.valueOf(lev);
		         
		         if (checkSSID(ssid))
		        	 wifis[i] = wifis[i] + ",Saved";
	         }
	         
	       if (ba.subExists(eventname.toLowerCase() + "_scandone"))
	         ba.raiseEventFromDifferentThread(this, null, 0, eventname.toLowerCase() + "_scandone", false, new Object[] {wifis, Results.size()});*/

	   }
	   
	   private class WifiScanReceiver extends BroadcastReceiver {
	      MLWifiScanner tScan;
	      
	      public WifiScanReceiver(MLWifiScanner tmpScan){
	      tScan = tmpScan;
	      }
	      
	      @Override
	      public void onReceive(Context c, Intent intent) {
	    	 String ssid = "";
	    	 String tmp = "";
	    	 tScan.Results = tScan.WifiObj.getScanResults();
	    	 
	    	 tScan.wifis = new String[tScan.Results.size()];
	         
	         for(int i = 0; i < tScan.Results.size(); i++){
	        	 wifis[i] = tScan.Results.get(i).SSID;
	        	 ssid = tScan.wifis[i];
	        	 tmp = tScan.Results.get(i).capabilities;
	        	 int lev = tScan.Results.get(i).level;
	        	 
	        	 if (tmp.contains("WPA2")) 
	        		 tScan.wifis[i] = tScan.wifis[i] + ",WPA2" + "," + String.valueOf(lev);
	        	 else
		         if (tmp.contains("WPA"))
		        	 tScan.wifis[i] = tScan.wifis[i] + ",WPA" + "," + String.valueOf(lev);
		        else
		        if (tmp.contains("WEP"))
		        	tScan.wifis[i] = tScan.wifis[i] + ",WEP" + "," + String.valueOf(lev);
		        else
		        	tScan.wifis[i] = tScan.wifis[i] + ",NONE" + "," + String.valueOf(lev);
		         
		         if (checkSSID(ssid))
		        	 tScan.wifis[i] = tScan.wifis[i] + ",Saved";
		         
		         //BA.Log(tScan.wifis[i]);
	          }
	         
	         //BA.Log(_ba.className);
		      if (_ba.subExists(tScan.eventname + "_scandone"))
		    	  _ba.raiseEventFromDifferentThread(tScan, null, 0, tScan.eventname + "_scandone", false, new Object[] {tScan.wifis, tScan.wifis.length});
		      
	      }
	     	      
	 }
	   
	   
	   private boolean checkSSID(String SSID){
		   Context context = BA.applicationContext;
		   int i = 0;
		   
		   WifiManager wm = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);              
		   List<WifiConfiguration> tmp = wm.getConfiguredNetworks();
		   
		   if (tmp !=null){
		   for (i = 0; i < tmp.size(); i++)
		      if (tmp.get(i).SSID.equals("\""+SSID+"\"")) {
		   
		         //BA.Log("SSID= " + tmp.get(i).SSID);
		         return true;
		      }
		   }
		   
		   return false;
	   }
	   
	   
	   /**
	    * Returns the BSSID for the selected entry. 
	    * 
	    * EntryNumber - If you display the string array wifis or the array returned in the getWifiList method UNSORTED, you can pass the Position variable from your ListView_Click event.
	    * Otherwise, you must pass the entry number from the string array since that is what is used as a reference.
	    */
	   public String getBSSID(int EntryNumber) {
		   
		   return Results.get(EntryNumber).BSSID;
		   
	   }
	   
	   /**
	    * Disconnects the current wifi AP connection
	    * 
	    * Returns True on success
	    */
	   public boolean Disconnect() {
		   Context context = BA.applicationContext;
		   WifiManager wifiManager = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);
		   
		   return wifiManager.disconnect();		   
	   }
	   
	   
	   /**
	    * Returns the LinkSpeed of the connected wifi access point
	    * 
	    */
	   public int getLinkSpeed() {
		   Context context = BA.applicationContext;
		   
		   WifiManager wifiManager = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);
		   WifiInfo wi = wifiManager.getConnectionInfo();
		   
		   return wi.getLinkSpeed();
	   }
	   
	   
	   /**
	    * Returns a String containing info about the entry. A startScan must have already been done. The string contains the SSID, the BSSID, and the capabilities of the network access point.
	    * Each item is separated by a comma (SSID,BSSID,CapabilityString). The capabilities part can be quite long.
	    * 
	    * EntryNumber - If you display the string array wifis or the array returned in the getWifiList method UNSORTED, you can pass the Position variable from your ListView_Click event.
	    * Otherwise, you must pass the entry number from the string array since that is what is used as a reference.
	    * 
	    * Returns a string of comma separated entries of SSID,BSSID, and capability string. Returns an empty string on error. 
	    */
	   public String WifiCap(int EntryNumber) {
		   String line = "";
		   
		   line = Results.get(EntryNumber).SSID + "," + Results.get(EntryNumber).BSSID + "," + Results.get(EntryNumber).capabilities;
		   
		   return line;
	   }
	   
	   
	   private WifiConfiguration wfc = new WifiConfiguration();
	   private String password = "";
	   
	   
	   /**
	    * Checks the stored networks on the device to see if this entry is a previously saved access point
	    * 
	    * EntryNumber - The entry number in the wifis array to check. If you display the string array wifis or the array returned in the getWifiList method UNSORTED, you can pass the Position variable from your ListView_Click event.
	    * Otherwise, you must pass the entry number from the string array since that is what is used as a reference.
	    * 
	    * Returns True if this is a stored network
	    */
	   public boolean isStored(int EntryNumber) {
		   Context context = BA.applicationContext;
		   int i = 0;
		   
		   WifiManager wm = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);              
		   List<WifiConfiguration> tmp = wm.getConfiguredNetworks();
		   
		   for (i = 0; i < tmp.size(); i++)
		      if (tmp.get(i).SSID.equals("\""+Results.get(EntryNumber).SSID+"\"")) {
		   
		         BA.Log("Stored SSID= " + tmp.get(i).SSID);
		         return true;
		      }
		   
		   return false;
	   }
	   
	   
	   /**
	    * Connects to a network already configured on the device. The device will use saved values to connect. Updated to handle quoted or unquoted SSID strings.
	    * 
	    * EntryNumber - If you display the string array wifis or the array returned in the getWifiList method UNSORTED, you can pass the Position variable from your ListView_Click event.
	    * Otherwise, you must pass the entry number from the string array since that is what is used as a reference.
	    * 
	    * Returns true if successful
	    */
	   public boolean connectToAP(int EntryNumber){
		   int i = 0;		   
		   Context context = BA.applicationContext;
		   int netId = -1;         
		   String ssid = "";
		   String bssid = "";
		   
		   WifiManager wm = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);              
		   
		   if(wm != null && wm.isWifiEnabled())
		   {
		   List<WifiConfiguration> tmp = wm.getConfiguredNetworks();
		   
		   if (!Results.get(EntryNumber).SSID.startsWith("\"")) 				   
		   ssid = "\""+Results.get(EntryNumber).SSID+"\"";
		   else
		   ssid = Results.get(EntryNumber).SSID;
		   
		   bssid = Results.get(EntryNumber).BSSID;
		   
		   BA.Log("Results SSID = " + ssid);
		   
		   try{
		
		   for (i = 0; i < tmp.size(); i++) {
			   BA.Log("Stored.ssid = " + tmp.get(i).SSID + " Searched ssid = " + ssid);
		   
			  if (tmp.get(i).SSID.equals(ssid) && tmp.get(i).BSSID.equals(bssid)) 
		      {
		         netId = tmp.get(i).networkId;             
		         //wm.disconnect();
		         wm.enableNetwork(netId, true);
		         //wm.reconnect();
		         
		         BA.Log("Connecting to " + tmp.get(i).SSID + " NetID = " + netId);
		         return true;
		      }
		   
		   }//for
		   
		   }//try
		   
		   catch(Exception e){
			   BA.Log(e.toString());
		   }
		   }
		   
		return false;
		   
	   }
	   
	   
	   /**
	    * Builds a list of all saved networks on the device.
	    * 
	    * Returns a comma separated list of SSID's and their BSSID and network ID number.
	    */
	   public List<String> listSavedNetworks(){
		   Context context = BA.applicationContext;
		   List<String> netlst = new ArrayList<String>();
		   
		   WifiManager wifiManager = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);              
		   
		   for (WifiConfiguration tmp : wifiManager.getConfiguredNetworks()) {
		      netlst.add(tmp.SSID + "," + tmp.BSSID + "," + String.valueOf(tmp.networkId));
		   }
		   
		   return netlst;
	   }
	   
	   
	   /**
	    * Adds an entry to the wifi manager and connects to it. This method saves the configuration of the new entry onto the device. Use the forgetAP method to delete them. Updated to handle quoted or unquoted SSID strings.
	    * 
	    * EntryNumber - If you display the string array wifis or the array returned in the getWifiList method UNSORTED, you can pass the Position variable from your ListView_Click event.
	    * Otherwise, you must pass the entry number from the string array since that is what is used as a reference.
	    * 
	    * securityType - The Access Point security type (returned in the scan result string array)
	    * 
	    * Password - The password/phrase for the wifi access point. Pass "" for an open network. Handles WEP HEX or text phrases
	    * 
	    * It is up to you to supply all information. The security type is found in the scan results array.
	    *  
	    * Returns True if the connection is successful
	    */
	   public boolean addAP(int EntryNumber, String securityType, String Password){
		   boolean isconnected = false;
		   Context context = BA.applicationContext;
		   password = Password;
		   WifiConfiguration wfc = new WifiConfiguration();

		   if (Results.get(EntryNumber).SSID.startsWith("\"")) 			
			   wfc.SSID = "\"".concat(Results.get(EntryNumber).SSID).concat("\"");
		   else
			   wfc.SSID = Results.get(EntryNumber).SSID;
		   
		   wfc.BSSID = Results.get(EntryNumber).BSSID;
		   
		   //wfc.SSID = "\"".concat(Results.get(EntryNumber).SSID).concat("\"");
		   wfc.status = WifiConfiguration.Status.DISABLED;
		   wfc.priority = 40;
		   
		   if (securityType.equals("NONE"))
			   doOpen();
		   else
		   if (securityType.equals("WEP"))
			   doWEP(password);
		   else
		   if (securityType.contains("WPA"))
			   doWPA(password);
		   else
			   return false;
		   
		   isconnected = false;
		   
		   WifiManager wfMgr = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
		   int networkId = wfMgr.addNetwork(wfc);
		   
		   if(wfMgr != null && wfMgr.isWifiEnabled())
		   {
		   if (networkId != -1) {
		    // success, can call wfMgr.enableNetwork(networkId, true) to connect
			   isconnected = wfMgr.enableNetwork(networkId, true);
			   
			   wfMgr.saveConfiguration();
		   } 
		   }
		   return isconnected;
	   }
	   
	   /**
	    * Forgets (erases) a stored access point on the device.
	    * 
	    * NetId - The network ID number to forget (returned in the listSavedNetworks method)
	    * 
	    * Returns True if successful
	    */
	   public boolean forgetAP(int NetId) {
		   Context context = BA.applicationContext;
		   boolean ok = false;
		   
		   WifiManager wm = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);
		   
		   if(wm != null && wm.isWifiEnabled())
		   {
		   ok = wm.removeNetwork(NetId);
		   wm.saveConfiguration();
		   }
		   
		   return ok;
	   }
	   
	   
	   private void doOpen(){
		   wfc.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.NONE);
		   wfc.allowedProtocols.set(WifiConfiguration.Protocol.RSN);
		   wfc.allowedProtocols.set(WifiConfiguration.Protocol.WPA);
		   wfc.allowedAuthAlgorithms.clear();
		   wfc.allowedPairwiseCiphers.set(WifiConfiguration.PairwiseCipher.CCMP);
		   wfc.allowedPairwiseCiphers.set(WifiConfiguration.PairwiseCipher.TKIP);
		   wfc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.WEP40);
		   wfc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.WEP104);
		   wfc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.CCMP);
		   wfc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.TKIP);
	   }
	   
	   private void doWEP(String password){
		   wfc.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.NONE);
		   wfc.allowedProtocols.set(WifiConfiguration.Protocol.RSN);
		   wfc.allowedProtocols.set(WifiConfiguration.Protocol.WPA);
		   wfc.allowedAuthAlgorithms.set(WifiConfiguration.AuthAlgorithm.OPEN);
		   wfc.allowedAuthAlgorithms.set(WifiConfiguration.AuthAlgorithm.SHARED);
		   wfc.allowedPairwiseCiphers.set(WifiConfiguration.PairwiseCipher.CCMP);
		   wfc.allowedPairwiseCiphers.set(WifiConfiguration.PairwiseCipher.TKIP);
		   wfc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.WEP40);
		   wfc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.WEP104);

		   if (isHex(password)) wfc.wepKeys[0] = password;
		   else wfc.wepKeys[0] = "\"".concat(password).concat("\"");
		   wfc.wepTxKeyIndex = 0;
	   }
	   
	   private void doWPA(String password){
		   wfc.allowedProtocols.set(WifiConfiguration.Protocol.RSN);
		   wfc.allowedProtocols.set(WifiConfiguration.Protocol.WPA);
		   wfc.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.WPA_PSK);
		   wfc.allowedPairwiseCiphers.set(WifiConfiguration.PairwiseCipher.CCMP);
		   wfc.allowedPairwiseCiphers.set(WifiConfiguration.PairwiseCipher.TKIP);
		   wfc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.WEP40);
		   wfc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.WEP104);
		   wfc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.CCMP);
		   wfc.allowedGroupCiphers.set(WifiConfiguration.GroupCipher.TKIP);

		   wfc.preSharedKey = "\"".concat(password).concat("\"");
	   }
	   
	   private boolean isHex (String in) {
	        boolean ret;
	        try {
	            // try to parse the string to an integer, using 16 as radix
	            @SuppressWarnings("unused")
				int t = Integer.parseInt(in, 16);
	            // parsing succeeded, string is valid hex number
	            ret = true;
	        } 
	        
	        catch (NumberFormatException e) {
	            // parsing failed, string is not a valid hex number
	            ret = false;
	        }
	        
	        return (ret);
	    }
	   	  
	   }

}


